@@ -7,11 +7,7 @@
 # Created on 2014-02-15
 #
 
-"""
-A lightweight HTTP library with a requests-like interface.
-"""
-
-from __future__ import print_function
+"""Lightweight HTTP library with a requests-like interface."""
 
 import codecs
 import json
@@ -27,10 +23,8 @@ import urllib2
 import urlparse
 import zlib
 
-import sys
 
-
-USER_AGENT = u'Alfred-Workflow/1.11 (http://www.deanishe.net)'
+USER_AGENT = u'Alfred-Workflow/1.19 (+http://www.deanishe.net/alfred-workflow)'
 
 # Valid characters for multipart form data boundaries
 BOUNDARY_CHARS = string.digits + string.ascii_letters
@@ -81,10 +75,12 @@ RESPONSES = {
 
 
 def str_dict(dic):
-    """Convert keys and values in ``dic`` into UTF-8-encoded :class:`str`
+    """Convert keys and values in ``dic`` into UTF-8-encoded :class:`str`.
 
-    :param dic: :class:`dict` of Unicode strings
-    :returns: :class:`dict`
+    :param dic: Mapping of Unicode strings
+    :type dic: dict
+    :returns: Dictionary containing only UTF-8 strings
+    :rtype: dict
 
     """
     if isinstance(dic, CaseInsensitiveDictionary):
@@ -101,7 +97,7 @@ def str_dict(dic):
 
 
 class NoRedirectHandler(urllib2.HTTPRedirectHandler):
-    """Prevent redirections"""
+    """Prevent redirections."""
 
     def redirect_request(self, *args):
         return None
@@ -109,9 +105,10 @@ class NoRedirectHandler(urllib2.HTTPRedirectHandler):
 
 # Adapted from https://gist.github.com/babakness/3901174
 class CaseInsensitiveDictionary(dict):
-    """
-    Dictionary that enables case insensitive searching while preserving
-    case sensitivity when keys are listed, ie, via keys() or items() methods.
+    """Dictionary with caseless key search.
+
+    Enables case insensitive searching while preserving case sensitivity
+    when keys are listed, ie, via keys() or items() methods.
 
     Works by storing a lowercase version of the key as the new key and
     stores the original key-value pair as the key's value
@@ -120,7 +117,7 @@ class CaseInsensitiveDictionary(dict):
     """
 
     def __init__(self, initval=None):
-
+        """Create new case-insensitive dictionary."""
         if isinstance(initval, dict):
             for key, value in initval.iteritems():
                 self.__setitem__(key, value)
@@ -176,7 +173,7 @@ class Response(object):
     """
     Returned by :func:`request` / :func:`get` / :func:`post` functions.
 
-    A simplified version of the ``Response`` object in the ``requests`` library.
+    Simplified version of the ``Response`` object in the ``requests`` library.
 
     >>> r = request('http://www.google.com')
     >>> r.status_code
@@ -191,14 +188,16 @@ class Response(object):
 
     """
 
-    def __init__(self, request):
+    def __init__(self, request, stream=False):
         """Call `request` with :mod:`urllib2` and process results.
 
         :param request: :class:`urllib2.Request` instance
+        :param stream: Whether to stream response or retrieve it all at once
+        :type stream: bool
 
         """
-
         self.request = request
+        self._stream = stream
         self.url = None
         self.raw = None
         self._encoding = None
@@ -207,6 +206,7 @@ class Response(object):
         self.reason = None
         self.headers = CaseInsensitiveDictionary()
         self._content = None
+        self._content_loaded = False
         self._gzipped = False
 
         # Execute query
@@ -244,24 +244,40 @@ class Response(object):
                     'gzip' in headers.get('transfer-encoding', '')):
                 self._gzipped = True
 
+    @property
+    def stream(self):
+        """Whether response is streamed.
+
+        Returns:
+            bool: `True` if response is streamed.
+        """
+        return self._stream
+
+    @stream.setter
+    def stream(self, value):
+        if self._content_loaded:
+            raise RuntimeError("`content` has already been read from "
+                               "this Response.")
+
+        self._stream = value
+
     def json(self):
         """Decode response contents as JSON.
 
         :returns: object decoded from JSON
-        :rtype: :class:`list` / :class:`dict`
+        :rtype: list, dict or unicode
 
         """
-
         return json.loads(self.content, self.encoding or 'utf-8')
 
     @property
     def encoding(self):
-        """Text encoding of document or ``None``
+        """Text encoding of document or ``None``.
 
-        :returns: :class:`str` or ``None``
+        :returns: Text encoding if found.
+        :rtype: str or ``None``
 
         """
-
         if not self._encoding:
             self._encoding = self._get_encoding()
 
@@ -269,13 +285,12 @@ class Response(object):
 
     @property
     def content(self):
-        """Raw content of response (i.e. bytes)
+        """Raw content of response (i.e. bytes).
 
         :returns: Body of HTTP response
-        :rtype: :class:`str`
+        :rtype: str
 
         """
-
         if not self._content:
 
             # Decompress gzipped content
@@ -286,6 +301,8 @@ class Response(object):
             else:
                 self._content = self.raw.read()
 
+            self._content_loaded = True
+
         return self._content
 
     @property
@@ -296,10 +313,9 @@ class Response(object):
         itself, the encoded response body will be returned instead.
 
         :returns: Body of HTTP response
-        :rtype: :class:`unicode` or :class:`str`
+        :rtype: unicode or str
 
         """
-
         if self.encoding:
             return unicodedata.normalize('NFC', unicode(self.content,
                                                         self.encoding))
@@ -311,12 +327,20 @@ class Response(object):
         .. versionadded:: 1.6
 
         :param chunk_size: Number of bytes to read into memory
-        :type chunk_size: ``int``
+        :type chunk_size: int
         :param decode_unicode: Decode to Unicode using detected encoding
-        :type decode_unicode: ``Boolean``
+        :type decode_unicode: bool
         :returns: iterator
 
         """
+        if not self.stream:
+            raise RuntimeError("You cannot call `iter_content` on a "
+                               "Response unless you passed `stream=True`"
+                               " to `get()`/`post()`/`request()`.")
+
+        if self._content_loaded:
+            raise RuntimeError(
+                "`content` has already been read from this Response.")
 
         def decode_stream(iterator, r):
 
@@ -328,8 +352,8 @@ class Response(object):
                     yield data
 
             data = decoder.decode(b'', final=True)
-            if data:
-                yield data  # pragma: nocover
+            if data:  # pragma: no cover
+                yield data
 
         def generate():
 
@@ -354,19 +378,20 @@ class Response(object):
         return chunks
 
     def save_to_path(self, filepath):
-        """Save retrieved data to file at ``filepath``
+        """Save retrieved data to file at ``filepath``.
 
         .. versionadded: 1.9.6
 
         :param filepath: Path to save retrieved data.
 
         """
-
         filepath = os.path.abspath(filepath)
         dirname = os.path.dirname(filepath)
         if not os.path.exists(dirname):
             os.makedirs(dirname)
 
+        self.stream = True
+
         with open(filepath, 'wb') as fileobj:
             for data in self.iter_content():
                 fileobj.write(data)
@@ -376,7 +401,6 @@ class Response(object):
 
         error will be instance of :class:`urllib2.HTTPError`
         """
-
         if self.error is not None:
             raise self.error
         return
@@ -385,10 +409,9 @@ class Response(object):
         """Get encoding from HTTP headers or content.
 
         :returns: encoding or `None`
-        :rtype: ``unicode`` or ``None``
+        :rtype: unicode or ``None``
 
         """
-
         headers = self.raw.info()
         encoding = None
 
@@ -401,20 +424,21 @@ class Response(object):
                 encoding = param[8:]
                 break
 
-        # Encoding declared in document should override HTTP headers
-        if self.mimetype == 'text/html':  # sniff HTML headers
-            m = re.search("""<meta.+charset=["']{0,1}(.+?)["'].*>""",
-                          self.content)
-            if m:
-                encoding = m.group(1)
+        if not self.stream:  # Try sniffing response content
+            # Encoding declared in document should override HTTP headers
+            if self.mimetype == 'text/html':  # sniff HTML headers
+                m = re.search("""<meta.+charset=["']{0,1}(.+?)["'].*>""",
+                              self.content)
+                if m:
+                    encoding = m.group(1)
 
-        elif ((self.mimetype.startswith('application/') or
-               self.mimetype.startswith('text/')) and
-              'xml' in self.mimetype):
-            m = re.search("""<?xml.+encoding=["'](.+?)["'][^>]*\?>""",
-                          self.content)
-            if m:
-                encoding = m.group(1)
+            elif ((self.mimetype.startswith('application/') or
+                   self.mimetype.startswith('text/')) and
+                  'xml' in self.mimetype):
+                m = re.search("""<?xml.+encoding=["'](.+?)["'][^>]*\?>""",
+                              self.content)
+                if m:
+                    encoding = m.group(1)
 
         # Format defaults
         if self.mimetype == 'application/json' and not encoding:
@@ -432,31 +456,35 @@ class Response(object):
 
 
 def request(method, url, params=None, data=None, headers=None, cookies=None,
-            files=None, auth=None, timeout=60, allow_redirects=False):
+            files=None, auth=None, timeout=60, allow_redirects=False,
+            stream=False):
     """Initiate an HTTP(S) request. Returns :class:`Response` object.
 
     :param method: 'GET' or 'POST'
-    :type method: ``unicode``
+    :type method: unicode
     :param url: URL to open
-    :type url: ``unicode``
+    :type url: unicode
     :param params: mapping of URL parameters
-    :type params: :class:`dict`
+    :type params: dict
     :param data: mapping of form data ``{'field_name': 'value'}`` or
         :class:`str`
-    :type data: :class:`dict` or :class:`str`
+    :type data: dict or str
     :param headers: HTTP headers
-    :type headers: :class:`dict`
+    :type headers: dict
     :param cookies: cookies to send to server
-    :type cookies: :class:`dict`
+    :type cookies: dict
     :param files: files to upload (see below).
-    :type files: :class:`dict`
+    :type files: dict
     :param auth: username, password
-    :type auth: ``tuple``
+    :type auth: tuple
     :param timeout: connection timeout limit in seconds
-    :type timeout: ``int``
+    :type timeout: int
     :param allow_redirects: follow redirections
-    :type allow_redirects: ``Boolean``
-    :returns: :class:`Response` object
+    :type allow_redirects: bool
+    :param stream: Stream content instead of fetching it all at once.
+    :type stream: bool
+    :returns: Response object
+    :rtype: :class:`Response`
 
 
     The ``files`` argument is a dictionary::
@@ -472,9 +500,7 @@ def request(method, url, params=None, data=None, headers=None, cookies=None,
       will be used.
 
     """
-
     # TODO: cookies
-    # TODO: any way to force GET or POST?
     socket.setdefaulttimeout(timeout)
 
     # Default handlers
@@ -510,6 +536,10 @@ def request(method, url, params=None, data=None, headers=None, cookies=None,
 
     headers['accept-encoding'] = ', '.join(encodings)
 
+    # Force POST by providing an empty data string
+    if method == 'POST' and not data:
+        data = ''
+
     if files:
         if not data:
             data = {}
@@ -538,41 +568,42 @@ def request(method, url, params=None, data=None, headers=None, cookies=None,
         url = urlparse.urlunsplit((scheme, netloc, path, query, fragment))
 
     req = urllib2.Request(url, data, headers)
-    return Response(req)
+    return Response(req, stream)
 
 
 def get(url, params=None, headers=None, cookies=None, auth=None,
-        timeout=60, allow_redirects=True):
+        timeout=60, allow_redirects=True, stream=False):
     """Initiate a GET request. Arguments as for :func:`request`.
 
     :returns: :class:`Response` instance
 
     """
-
     return request('GET', url, params, headers=headers, cookies=cookies,
-                   auth=auth, timeout=timeout, allow_redirects=allow_redirects)
+                   auth=auth, timeout=timeout, allow_redirects=allow_redirects,
+                   stream=stream)
 
 
 def post(url, params=None, data=None, headers=None, cookies=None, files=None,
-         auth=None, timeout=60, allow_redirects=False):
+         auth=None, timeout=60, allow_redirects=False, stream=False):
     """Initiate a POST request. Arguments as for :func:`request`.
 
     :returns: :class:`Response` instance
 
     """
     return request('POST', url, params, data, headers, cookies, files, auth,
-                   timeout, allow_redirects)
+                   timeout, allow_redirects, stream)
 
 
 def encode_multipart_formdata(fields, files):
     """Encode form data (``fields``) and ``files`` for POST request.
 
     :param fields: mapping of ``{name : value}`` pairs for normal form fields.
-    :type fields: :class:`dict`
+    :type fields: dict
     :param files: dictionary of fieldnames/files elements for file data.
                   See below for details.
-    :type files: :class:`dict` of :class:`dicts`
-    :returns: ``(headers, body)`` ``headers`` is a :class:`dict` of HTTP headers
+    :type files: dict of :class:`dict`
+    :returns: ``(headers, body)`` ``headers`` is a
+        :class:`dict` of HTTP headers
     :rtype: 2-tuple ``(dict, str)``
 
     The ``files`` argument is a dictionary::
@@ -583,17 +614,18 @@ def encode_multipart_formdata(fields, files):
         }
 
     - ``fieldname`` is the name of the field in the HTML form.
-    - ``mimetype`` is optional. If not provided, :mod:`mimetypes` will be used to guess the mimetype, or ``application/octet-stream`` will be used.
+    - ``mimetype`` is optional. If not provided, :mod:`mimetypes` will
+      be used to guess the mimetype, or ``application/octet-stream``
+      will be used.
 
     """
-
     def get_content_type(filename):
         """Return or guess mimetype of ``filename``.
 
         :param filename: filename of file
-        :type filename: unicode/string
+        :type filename: unicode/str
         :returns: mime-type, e.g. ``text/html``
-        :rtype: :class::class:`str`
+        :rtype: str
 
         """
 
